/*------------------------------------------------------------------------------
 bin/djv_view/app.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "app.h"
#include <djv_base/file.h>
#include "prefs.h"
#include "prefs_dialog.h"
#include "cache.h"
#include "window.h"
#include "image_color.h"
#include "magnify.h"
#include "color_sample.h"
#include "histogram.h"
#include "image_info.h"
#include <FL/Fl.H>

App::App(int & argc, char ** argv) :
  ui::app::App("djv_view", argc, argv)
{
  DJV_DBG_OBJ("App::App");
  
  // Command line.
  
  bool r = false;
  try { r = cmdln(argc, argv); }
  catch (Error in) { error(str::Format("Command Line: %1").arg(in)); }
  if (! r) { djv::app::App::exit(1); return; }

  List<Str> input;
  for (int i = 1; i < argc; ++i) input += argv[i];
  
  // Initialize.
  
  valid(true);

  prefs::init();
  prefs_dialog::init();

  if (input.size()) {
    for (uint i = 0; i < input.size(); ++i) {
    
      File file(input[i]);
      if (file.seq_valid()) file.type = file::SEQ;
    
      window::Widget * window = window::factory()->add();
      window->file_open(file);
      window->show();
    }
  } else {

    window::Widget * window = window::factory()->add();
    window->show();
  }
}

App::~App()
{
  DJV_DBG_OBJ("App::~App");
  
  window::factory_del();
  image_color::del();
  magnify::del();
  color_sample::del();
  histogram::del();
  image_info::del();
  cache::del();
  prefs::del();
}

Str App::help() const
{
  return str::Format(
"\n"
" djv_view\n"
" --------\n"
" Image and movie viewing application.\n"
"\n"
" Usage\n"
" -----\n"
" djv_view [image]... [option]...\n"
"\n"
"%1"
  ).
    arg(djv::app::App::help());
}

int main(int argc, char ** argv)
{
  int r = 1;
  try { r = App(argc, argv).exec(); }
  catch (Error in) { error::print(in); }
  return r;
}

