/*------------------------------------------------------------------------------
 bin/djv_view/image_info.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "image_info.h"
#include <djv_ui/prefs.h>
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/text.h>

namespace image_info {

Dialog::Dialog() :
  Base("Information"),
  _window(0),
  _update(false)
{
  // Widgets.

  _file_widget = new ui::text::Display;
  ui::label::Widget * file_label = new ui::label::Widget("File:");
  _width_widget = new ui::text::Display;
  ui::label::Widget * width_label = new ui::label::Widget("Width:");
  _height_widget = new ui::text::Display;
  ui::label::Widget * height_label = new ui::label::Widget("Height:");
  _aspect_widget = new ui::text::Display;
  ui::label::Widget * aspect_label = new ui::label::Widget("Aspect:");
  _pixel_widget = new ui::text::Display;
  ui::label::Widget * pixel_label = new ui::label::Widget("Pixel:");
  _tag_widget = new ui::text::Multiline_Display;
  ui::label::Widget * tag_label = new ui::label::Widget("Tags:");
  
  ui::button::Push * close_button = new ui::button::Push("Clos&e");
  
  // Layout.
  
  ui::layout::group(List<ui::layout::Item *>() << file_label << width_label <<
    pixel_label);
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(file_label);
  h_layout->add(_file_widget);
  
  h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(width_label);
  h_layout->add(_width_widget);
  h_layout->add(height_label);
  h_layout->add(_height_widget);
  h_layout->add(aspect_label);
  h_layout->add(_aspect_widget);
  
  h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(pixel_label);
  h_layout->add(_pixel_widget);
  
  layout->add_spacer(ui::style::style()->spacing_large());
  layout->add(tag_label);
  layout->add(_tag_widget);
  layout->stretch(_tag_widget);
  
  h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add_stretch();
  h_layout->add(close_button);
  
  // Initialize.
  
  size(Vec2i(300, 300));
  close_button->take_focus();
  
  // Callbacks.
 
  close_button->sig.set(this, close_callback);
}

void Dialog::set(const void * in, const Info & info)
{
  if (in == _window) return;
  _window = in;
  _info = info;
  if (shown() && visible()) widget_update();
  else _update = true;
}

void Dialog::update(const void * in, const Info & info)
{
  if (in != _window || ! shown() || ! visible()) return;
  _info = info;
  widget_update();
}

void Dialog::del(const void * in)
{
  if (in != _window) return;
  _window = 0;
}

void Dialog::show()
{
  if (_update) widget_update();
  ui::dialog::Base::show();
}

void Dialog::close_callback(bool)
{
  hide();
}

void Dialog::widget_update()
{
  callbacks(false);
  
  _file_widget->set(_info.file);
  
  _width_widget->set(str::int_to_str(_info.size.x));
  _height_widget->set(str::int_to_str(_info.size.y));
  _aspect_widget->set(str::float_to_str(vec::aspect(_info.size)));
  
  _pixel_widget->set(str::Format("%1").
    arg(image::pixel::pixel_label()[_info.pixel]));

  Str tmp;
  const List<Str> key_list = _info.tag.key_list();
  for (uint i = 0; i < key_list.size(); ++i)
    tmp += str::Format("%1 = %2\n").
      arg(key_list[i]).arg(_info.tag[key_list[i]]);
  _tag_widget->set(tmp);

  callbacks(true);
}

namespace { std::auto_ptr<Dialog> _dialog; }

Dialog * dialog()
{
  if (! _dialog.get()) _dialog = std::auto_ptr<Dialog>(new Dialog);
  return _dialog.get();
}

void del()
{
  if (_dialog.get()) _dialog->widget_del();
  _dialog.release();
}

}

