/*------------------------------------------------------------------------------
 bin/djv_view/magnify.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "magnify.h"
#include <djv_ui/prefs.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include "prefs.h"
#include "view.h"

namespace magnify {

Dialog::Dialog() :
  Base("Magnify"),
  _view(0),
  _zoom(1),
  _update(false)
{
  //DJV_DBG_OBJ("magnify::Dialog::Dialog");
  
  // Widgets.

  _widget = new ui::image::Widget;
  _widget->background(prefs::view()->background());
  
  _zoom_widget = new ui::choice::Widget(zoom_str());
  
  ui::button::Push * close_button = new ui::button::Push("Clos&e");
  
  ui::group::Frame * frame = new ui::group::Frame;
  frame->frame(FL_THIN_DOWN_BOX);

  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  
  layout->add(frame);
  layout->stretch(frame);
  ui::layout::Vertical * v_layout = new ui::layout::Vertical(frame);
  v_layout->margin(0);
  v_layout->add(_widget);
  
  ui::layout::Horizontal * h_layout = new ui::layout::Horizontal(layout);
  h_layout->margin(0);
  h_layout->add(_zoom_widget);
  h_layout->add_spacer(-1, true);
  h_layout->add(close_button);
  
  // Initialize.
  
  ui::prefs::Prefs prefs(ui::prefs::app(), "magnify");
  List<Str> tmp;
  if (prefs.get("zoom", &tmp)) str::serialize(&tmp, &_zoom);

  widget_update();

  size(max(size_hint(), Vec2i(300, 300)));
  close_button->take_focus();
  
  // Callbacks.

  _zoom_widget->sig.set(this, zoom_callback);
  close_button->sig.set(this, close_callback);
}

void Dialog::widget_del()
{
  ui::prefs::Prefs prefs(ui::prefs::app(), "magnify");
  prefs.set("zoom", str::serialize(_zoom));

  Base::widget_del();
}

void Dialog::set(const view::Widget * in, const Vec2i & pick)
{
  if (in == _view) return;
  _view = in;
  _pick = pick;
  if (shown() && visible()) widget_update();
  else _update = true;
}

void Dialog::update(const view::Widget * in, const Vec2i & pick)
{
  if (in != _view || ! shown() || ! visible()) return;
  _pick = pick;
  widget_update();
}

void Dialog::del(const view::Widget * in)
{
  if (in != _view) return;
  _view = 0;
  _widget->set(0);
  _widget->redraw();
}

void Dialog::show()
{
  if (_update) widget_update();
  ui::dialog::Base::show();
}

void Dialog::zoom_callback(int in)
{
  _zoom = in;
  widget_update();
}

void Dialog::close_callback(bool) { hide(); }

void Dialog::widget_update()
{
  if (! _view) return;

  DJV_DBG_OBJ("magnify::Dialog::widget_update");
  DJV_DBG_PRN("pick = " << _pick);
  
  callbacks(false);

  const double zoom = math::pow(2, _zoom + 1);
  DJV_DBG_PRN("zoom = " << zoom);
  
  _widget->set(_view->get());
  _widget->view(-vec::to_int(vec::to_float(_pick) * zoom) +
    _widget->geom().size / 2);
  _widget->zoom(_view->zoom() * zoom);
  _widget->mirror(_view->mirror_x(), _view->mirror_y());
  _widget->scale(_view->scale());
  _widget->rotate(_view->rotate());
  _widget->profile(_view->profile());
  _widget->display(_view->display());
  _widget->channel(_view->channel());
  _widget->filter(_view->filter());
  _widget->redraw();
  
  _zoom_widget->set(_zoom);

  callbacks(true);
}

const List<Str> & zoom_str()
{
  static const List<Str> data = List<Str>() << "200%" << "400%" << "800%" <<
    "1600%";
  return data;
}

namespace { std::auto_ptr<Dialog> _dialog; }

Dialog * dialog()
{
  if (! _dialog.get()) _dialog = std::auto_ptr<Dialog>(new Dialog);
  return _dialog.get();
}

void del()
{
  if (_dialog.get()) _dialog->widget_del();
  _dialog.release();
}

}

