/*------------------------------------------------------------------------------
 bin/djv_view/prefs_dialog.h

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_PREFS_DIALOG_H
#define DJV_VIEW_PREFS_DIALOG_H

#include <djv_ui/widget.h>
#include "base.h"

namespace prefs_dialog {

// File preferences.

class File : public ui::widget::Widget {
public:

  File();
  
private:

  DJV_CALLBACK(File, proxy_callback, int);

  DJV_CALLBACK(File, cache_callback, bool);
  DJV_CALLBACK(File, cache_size_callback, int);
  DJV_CALLBACK(File, cache_menu_callback, int);
};

// Window preferences.

class Window : public ui::widget::Widget {
public:

  Window();

private:

  DJV_CALLBACK(Window, resize_callback, bool);
  DJV_CALLBACK(Window, controls_callback, bool);
};

// View preferences.

class View : public ui::widget::Widget {
public:

  View();

private:

  DJV_CALLBACK(View, grid_callback, int);
  DJV_CALLBACK(View, grid_callback, const image::Color &);
  DJV_CALLBACK(View, hud_callback, bool);
  DJV_CALLBACK(View, hud_foreground_callback, const image::Color &);
  DJV_CALLBACK(View, hud_background_callback, const image::Color &);
  DJV_CALLBACK(View, background_callback, const image::Color &);
  
  ui::color::Swatch * _grid_swatch;
  ui::color::Swatch * _hud_foreground_swatch, * _hud_background_swatch;
  ui::color::Swatch * _background_swatch;
};

// Image preferences.

class Image : public ui::widget::Widget {
public:

  Image();
  
private:

  DJV_CALLBACK(Image, mirror_x_callback, bool);
  DJV_CALLBACK(Image, mirror_y_callback, bool);
  DJV_CALLBACK(Image, scale_callback, int);
  DJV_CALLBACK(Image, rotate_callback, int);
  DJV_CALLBACK(Image, color_callback, int);
  DJV_CALLBACK(Image, color_list_callback, bool);
  DJV_CALLBACK(Image, channel_callback, int);
  DJV_CALLBACK(Image, filter_callback, int);
  
  void widget_update();
  
  ui::choice::Widget * _color_widget;
};

// Playback preferences.

class Playback : public ui::widget::Widget {
public:

  Playback();

private:

  DJV_CALLBACK(Playback, start_callback, bool);
  DJV_CALLBACK(Playback, speed_callback, const image::time::Speed &);
  DJV_CALLBACK(Playback, every_frame_callback, bool);
};

// Global.

void init();

}

#endif

