/*------------------------------------------------------------------------------
 lib/djv_base/io.h
 
 File I/O

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IO_H
#define DJV_IO_H

#include <djv_base/base.h>

namespace djv {
namespace io {

enum MODE { READ, WRITE };
enum POSITION { SET, OFFSET };

class Io {
public:

  Io();
  ~Io();

  // Open.
  
  void open(const Str &, MODE) throw (Error);
  void close();
  
  bool is_valid() const;

  // I/O.

  void set(const void *, size_t, size_t word_size = 1) throw (Error);
  void get(void *, size_t, size_t word_size = 1) throw (Error);

  void set_8(const int8_t *, size_t) throw (Error);
  void set_u8(const uint8_t *, size_t) throw (Error);
  void set_16(const int16_t *, size_t) throw (Error);
  void set_u16(const uint16_t *, size_t) throw (Error);
  void set_32(const int32_t *, size_t) throw (Error);
  void set_u32(const uint32_t *, size_t) throw (Error);
  void set_f32(const float *, size_t) throw (Error);

  void set_8(const int8_t & in) throw (Error) { return set_8(&in, 1); }
  void set_u8(const uint8_t & in) throw (Error) { return set_u8(&in, 1); }
  void set_16(const int16_t & in) throw (Error) { return set_16(&in, 1); }
  void set_u16(const uint16_t & in) throw (Error) { return set_u16(&in, 1); }
  void set_32(const int32_t & in) throw (Error) { return set_32(&in, 1); }
  void set_u32(const uint32_t & in) throw (Error) { return set_u32(&in, 1); }
  void set_f32(const float & in) throw (Error) { return set_f32(&in, 1); }
  
  void get_8(int8_t *, size_t = 1) throw (Error);
  void get_u8(uint8_t *, size_t = 1) throw (Error);
  void get_16(int16_t *, size_t = 1) throw (Error);
  void get_u16(uint16_t *, size_t = 1) throw (Error);
  void get_32(int32_t *, size_t = 1) throw (Error);
  void get_u32(uint32_t *, size_t = 1) throw (Error);
  void get_f32(float *, size_t = 1) throw (Error);

  // XXX Ill but fast.

  const uint8_t * mmap_p() const { return _mmap_p; }
  const uint8_t * mmap_end() const { return _mmap_end; }

  // Position.

  void position(off_t, POSITION = SET) throw (Error);
  off_t position() const;
  
  // Size.
  
  size_t size() const { return _size; }
  
  // Endian.

  void endian(bool);
  bool endian() const { return _endian; }

private:

#if defined(WIN32)
  HANDLE _f;
#else
  int _f;
#endif
  MODE _mode;
  size_t _size;
  bool _endian;
  void * _mmap;
  const uint8_t * _mmap_start, * _mmap_end, * _mmap_p;
  int _mmap_readahead;
};

// Functions.

void word(Io *, char *, int max_len = str::cstr_size) throw (Error);

// Labels.

static const Str
  error_namespace = "I/O",
  error_open = "File Open",
  error_mmap = "File Memory-Map",
  error_read = "File Read",
  error_write = "File Write",
  error_position = "File Position";

}
}

#endif

