/*------------------------------------------------------------------------------
 lib/djv_base/str_fnc.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "str.h"
#include <djv_base/dbg.h>
#include <djv_base/mem.h>
#include <djv_base/math.h>
#include <stdlib.h>

namespace djv {
namespace str {

List<Str> split(const Str & in, const Str & separator, bool keep_empty)
{
  List<Str> out;
  
  out += Str();
  int current = 0;
  for (uint i = 0; i < in.size(); ++i) {
    uint j = 0;
    for (; j < separator.size(); ++j)
      if (separator[j] == in[i]) {
        if (keep_empty || (! keep_empty && out[current].size())) {
          out += Str();
          ++current;
        }
      break;
      }
    if (j < separator.size()) continue;
    out[current] += in[i];
  }

  if (! keep_empty)
    if (out.size())
      if (! out[out.size() - 1].size())
        out.pop_back();

  return out;
}

List<Str> format(const Str & in, int width)
{
  width = math::max(width, 1);
  
  List<Str> out;
  
  const List<Str> line_list = str::split(in, "\n", true);
  for (uint i = 0; i < line_list.size(); ++i) {

    const List<Str> word_list = str::split(line_list[i], " ", true);
    for (uint j = 0; j < word_list.size();) {

      Str line;
      while (j < word_list.size()) {
        line += (line.size() ? " " : "") + word_list[j++];
        if (j < word_list.size())
          if (int(line.size() + 1 + word_list[j].size()) > width) break;
      }
      out += line;
    }
  }
  
  return out;
}

bool compare(const Str & a, const Str & b, bool case_insensitive)
{
  return case_insensitive ? (lower(a) == lower(b)) : (a == b);
}

int ascii(const Str & in, char * out, int max_len, bool terminate)
{
  int i = 0;
  const int size = max_len ?
    math::min(int(in.size()), max_len - int(terminate)) :
    int(in.size() + int(terminate));
  for (; i < size; ++i)
    out[i] = in[i];
  if (terminate) out[i++] = 0;
  return i;
}

double str_to_float(const Str & in)
{
  return atof(in.c_str());
}

int int_to_str(int in, char * out, int max_len)
{
  const int max = max_len - 1;
  const bool negative = in < 0;
  if (in < 0) in = -in;
  
  int size = 0;
  int tmp = in;
  do {
    ++size;
    tmp /= 10;
  } while (tmp && size < max);
  
  if (negative) {
    ++size;
    out[0] = '-';
  }
  for (int i = size - 1; i >= int(negative); --i, in /= 10)
    out[i] = '0' + (in % 10);
  out[size] = 0;

  return size;
}

Str int_to_str(int in)
{
  char tmp [cstr_size] = "";
  //const int size = SNPRINTF(tmp, cstr_size, "%d", in);
  const int size = int_to_str(in, tmp);
  return Str(tmp, size);
}

Str float_to_str(double in, int precision)
{  
  char tmp [cstr_size] = "";
  int size = 0;
  
  if (precision) {
    char format [] = "%0. f";
    format[3] = '0' + abs(precision);
    size = SNPRINTF(tmp, cstr_size, format, in);
  } else {
    size = SNPRINTF(tmp, cstr_size, "%0.3f", in);
  }

  if (precision <= 0 && size) {
    int i = size - 1;
    for (; i >= 0 && '0' == tmp[i]; --i, --size) ;
    if (i)
      if ('.' == tmp[i]) --size;
  }
  return Str(tmp, (size > 0) ? size : 0);
}

const List<Str> & bool_label()
{
  static const List<Str> data = List<Str>() << "False" << "True";
  return data;
}

}
}

