/*------------------------------------------------------------------------------
 lib/djv_image/data.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/data.cxx"

#include "data.h"
#include <djv_image/color.h>

namespace djv {
namespace image {
namespace data {

Data::Data()
{
  DJV_DBG_OBJ2(_DBG, "Data::Data");
  
  init();
}

Data::Data(const Data & in)
{
  DJV_DBG_OBJ2(_DBG, "Data::Data");

  init();
  set(in);
}

Data::Data(const Info & in, const uint8_t * data)
{
  DJV_DBG_OBJ2(_DBG, "Data::Data");

  init();
  set(in, data);
}

void Data::init()
{
  _channels = 0;
  _pixel_bytes = 0;
  _scanline_bytes = 0;
  _data_bytes = 0;
  _p = 0;
}

Data & Data::operator = (const Data & in)
{
  if (&in != this) set(in);
  return *this;
}

void Data::set(const Data & in)
{
  set(in._info);
  mem::copy(in.data(), data(), _data_bytes);
}

void Data::set(const Info & in, const uint8_t * data)
{
  DJV_DBG_OBJ2(_DBG, "Data::Data");
  DJV_DBG_PRN("in = " << in);

  _info = in;
  
  _channels = pixel::channels(_info.pixel);
  _pixel_bytes = pixel::bytes(_info.pixel);
  _scanline_bytes = data::scanline_bytes(_info);
  _data_bytes = data::data_bytes(_info);
  DJV_DBG_PRN("channels = " << _channels);
  DJV_DBG_PRN("pixel bytes = " << _pixel_bytes);
  DJV_DBG_PRN("scanline bytes = " << _scanline_bytes);
  DJV_DBG_PRN("data bytes = " << _data_bytes);

  if (data) {
    _data.size(0);
    _p = data;
  } else {
    _data.size(_data_bytes);
    _p = _data();
  }
}

void Data::zero()
{ 
  DJV_DBG_OBJ2(_DBG, "Data::zero");

  _data.zero();
}

}
}
}

