/*------------------------------------------------------------------------------
 lib/djv_image/io_dpx_color_space.cxx
 
 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_image/io_dpx_color_space.cxx"

#include "io_dpx.h"

namespace djv {
namespace image {
namespace io {
namespace dpx {

// XXX Preserve black and white points?

void print(pixel::F32_T * in, int size, const Print & value)
{
  DJV_DBG_OBJ2(_DBG, "print");
  DJV_DBG_PRN("size = " << size);
  DJV_DBG_PRN("black = " << value.black);
  DJV_DBG_PRN("white = " << value.white);
  DJV_DBG_PRN("gamma = " << value.gamma);
  
  const double gain = 1.0 / (1.0 - math::pow(
    math::pow(10.0, (value.black - value.white) * 0.002 / 0.6),
    value.gamma / 1.7));
  const double offset = gain - 1.0;
  DJV_DBG_PRN("gain = " << gain * 255);
  DJV_DBG_PRN("offset = " << offset * 255);
  
  for (int i = 0; i < size; ++i)
    in[i] = i / pixel::F32_T(size - 1);
  
  for (int i = 0; i < size; ++i) {

    in[i] = pixel::F32_T(
      value.white / 1023.0 +
      math::log_10(math::pow((in[i] + offset) / gain, 1.7 / value.gamma)) /
      (2.048 / 0.6));

    //DJV_DBG_PRN("lut[" << i << "] = " << in[i] << " " << int(in[i] * 1024));
  }
}

void unprint(pixel::F32_T * in, int size, const Unprint & value)
{
  DJV_DBG_OBJ2(_DBG, "unprint");
  DJV_DBG_PRN("black = " << value.black);
  DJV_DBG_PRN("white = " << value.white);
  DJV_DBG_PRN("gamma = " << value.gamma);
  DJV_DBG_PRN("soft clip = " << value.soft_clip);
  
  const double gain = 1.0 / (1.0 - math::pow(
    math::pow(10.0, (value.black - value.white) * 0.002 / 0.6),
    value.gamma / 1.7));
  const double offset = gain - 1.0;

  int break_point = value.white - value.soft_clip;
  double knee_offset = math::pow(
    math::pow(10.0, (break_point - value.white) * 0.002 / 0.6),
    value.gamma / 1.7) * gain - offset;
  double knee_gain = ((255 - (knee_offset * 255)) /
    math::pow(5.0 * value.soft_clip, value.soft_clip / 100.0)) / 255.0;
  
  DJV_DBG_PRN("break point = " << break_point);
  DJV_DBG_PRN("gain = " << gain * 255);
  DJV_DBG_PRN("offset = " << offset * 255);
  DJV_DBG_PRN("knee offset = " << knee_offset * 255);
  DJV_DBG_PRN("knee gain = " << knee_gain * 255);
  
  for (int i = 0; i < size; ++i)
    in[i] = i / pixel::F32_T(size - 1);
  
  for (int i = 0; i < size; ++i) {
    
    const int tmp = int(in[i] * 1023.0);
    
    if (tmp < value.black) {
      
      in[i] = 0.0;
      
    } else if (tmp > break_point) {
      
      in[i] = pixel::F32_T(
        (math::pow(double(tmp - break_point), value.soft_clip / 100.0) *
        knee_gain * 255 + knee_offset * 255) / 255.0);

    } else {

      in[i] = pixel::F32_T(
        math::pow(math::pow(10.0, (tmp - value.white) * 0.002 / 0.6),
        value.gamma / 1.7) * gain - offset);
    }

    //DJV_DBG_PRN("lut[" << i << "] = " << in[i] << " " << int(in[i] * 255.0));
  }
}

Data print_lut(const Print & in)
{
  Data out(data::Info(Vec2i(1024, 1), pixel::L_F32));
  print(reinterpret_cast<pixel::F32_T *>(out.data()), out.w(), in);
  return out;
}

Data unprint_lut(const Unprint & in)
{
  Data out(data::Info(Vec2i(1024, 1), pixel::L_F32));
  unprint(reinterpret_cast<pixel::F32_T *>(out.data()), out.w(), in);
  return out;
}

}
}
}
}
