/*------------------------------------------------------------------------------
 lib/djv_image/io_tga.h

 TARGA Image I/O
 
 References:
 * James D. Murray, William vanRyper, "Encyclopedia of Graphics File Formats,
   Second Edition"

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_TGA_H
#define DJV_IMAGE_IO_TGA_H

#include <djv_image/io.h>

namespace djv {
namespace image {
namespace io {
namespace tga {

// Information.

static const Str plugin_name = "TGA";
static const List<Str> extension_list = List<Str>() << ".tga";

// Options.

enum COMPRESS { COMPRESS_NONE, COMPRESS_RLE };

// Load.

class Load : public io::Load {
public:

  Load();

  Str name() const { return tga::plugin_name; }
  const List<Str> & extension_list() const { return tga::extension_list; }
  
  Plugin * copy() const { return new Load; }
  
  void open(const File &, io::Info *) throw (Error);
  const Image * get(int frame, int proxy) throw (Error);

private:

  void _open(const File &, io::Info *) throw (Error);

  File _file;
  djv::Io _io;
  bool _compress;
  Image _image, _image_tmp;
};

// Save.

class Save : public io::Save {
public:

  Save();
  
  Str name() const { return tga::plugin_name; }
  const List<Str> & extension_list() const { return tga::extension_list; }
  Str help() const;
  
  Plugin * copy() const;
  
  struct Options {
    Options() : compress(COMPRESS_NONE) {}
    COMPRESS compress;
  };
  enum OPTION { OPTION_COMPRESS };
  bool option(const Str &, List<Str> *);
  List<Str> option(const Str &) const;
  const List<Str> & option_list() const;
  
  void open(const File &, const io::Info &) throw (Error);
  void set(const Image &, int frame) throw (Error);

private:
  
  void _open(const File &) throw (Error);

  Options _options;
  File _file;
  djv::Io _io;
  Image _image;
};

// Functions.

void rle_unpack(djv::Io *, void *, int size, int channels) throw (Error);
size_t rle_pack(const void * in, void * out, int size, int channels);

// Labels.

const List<Str> & compress_label();
const List<Str> & save_option_label();

}
}
}
}

#endif
