/*------------------------------------------------------------------------------
 lib/djv_ui/base.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
#define _DBG "lib/djv_ui/base.cxx"

#include "base.h"
#include <djv_base/math.h>
#include <djv_image/data.h>
#include <djv_image/gl_data.h>
#include <FL/Fl.H>
#include <FL/Fl_Image.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {

Callback_Base::~Callback_Base() {}

void Callback_Base::del()
{
  for (uint i = 0; i < _sig_list.size(); ++i)
    _sig_list[i]->release(this);
  _sig_list.clear();
}

void Callback_Base::sig_add(Signal_Base * in) { _sig_list += in; }

void Callback_Base::sig_del(Signal_Base * in)
{
  List<Signal_Base *>::iterator i = std::find(
    _sig_list.begin(), _sig_list.end(), in);
  if (i != _sig_list.end()) _sig_list.erase(i);
}

Cursor::Cursor(const Fl_Cursor & in) { cursor(in); }
Cursor::~Cursor() { cursor(FL_CURSOR_DEFAULT); }

void cursor(const Fl_Cursor & in)
{
  fl_cursor(in);

  // XXX
  
  Fl::flush();
}

Vec2i str_size(const Str & in)
{
  int width = 0;
  const List<Str> tmp = str::split(in, "\n", true);
  for (uint i = 0; i < tmp.size(); ++i)
    width = math::max(str_width(tmp[i]), width);
  return Vec2i(width, tmp.size() * fl_height());
}

int str_width(const Str & in)
{
  return math::ceil(fl_width(in.c_str()));
}

void recent(const Str & in, List<Str> * list, int max)
{
  List<Str>::iterator i = std::find(list->begin(), list->end(), in);
  if (i == list->end()) {
    list->insert(list->begin(), in);
    while (int(list->size()) > max)
      list->erase(list->end() - 1);
  } else {
    const Str tmp = *i;
    list->erase(i);
    list->insert(list->begin(), tmp);
  }
}

Fl_Image * image_to_fl(const djv::image::Data & in)
{
  DJV_DBG_OBJ2(_DBG, "image_to_fl");
  DJV_DBG_PRN("in = " << in);
  
  djv::image::Data tmp(djv::image::data::Info(in.info().size,
    djv::image::pixel::RGB_U8));
  DJV_DBG_PRN("tmp = " << tmp);

  djv::image::gl::data::Options options;
  options.mirror_y = true;
  djv::image::gl::data::copy(in, &tmp, options);
  
  return Fl_RGB_Image(tmp.data(), tmp.w(), tmp.h()).copy();
}

}
}

