/*------------------------------------------------------------------------------
 lib/djv_ui/layout_item.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "layout.h"

namespace djv {
namespace ui {
namespace layout {

Item::Item(TYPE type) :
  _type(type),
  _parent(0)
{
  _size_policy[0] = _size_policy[1] = STRETCH;
  _align_policy[0] = _align_policy[1] = CENTER;
}

Item::~Item() {}

void Item::update()
{
  if (_parent) _parent->update();
}

void Item::resize(int x, int y, int w, int h)
{
  _geom = Box2i(x, y, w, h);
}

void Item::size_policy(SIZE_POLICY h, SIZE_POLICY v)
{
  _size_policy[0] = h;
  _size_policy[1] = v;
}

void Item::align_policy(ALIGN_POLICY h, ALIGN_POLICY v)
{
  _align_policy[0] = h;
  _align_policy[1] = v;
}

void Item::group(const List<Item *> & in)
{
  _group = in;
}

Vec2i Item::group_hint() const
{
  Vec2i out = size_hint();
  for (uint i = 0; i < _group.size(); ++i)
    if (_group[i]->visible() && this != _group[i])
      out.x = math::max(out.x, _group[i]->size_hint().x);
  return out;
}

void Item::parent(Item * in)
{
  _parent = in;
}

}
}
}

