/*------------------------------------------------------------------------------
 lib/djv_ui/num_int.h
 
 Integer Value Widgets

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_NUM_INT_H
#define DJV_UI_NUM_INT_H

#include <djv_ui/num.h>

namespace djv {
namespace ui {
namespace num {

// Integer edit widget.

class Int_Edit : public widget::Widget, public Int_Base {
public:

  Int_Edit();
  virtual ~Int_Edit() {}
  
  // Widget.
  
  virtual int handle(int);
  
  // Layout.
  
  virtual Item * group_align();
  
private:

  DJV_CALLBACK(Int_Edit, widget_callback, const Str &);
  DJV_CALLBACK(Int_Edit, inc_callback, bool);
  DJV_CALLBACK(Int_Edit, dec_callback, bool);
  
  void value_update();
  
  text::Edit * _widget;
};

// Integer display widget.

class Int_Display : public widget::Widget
{
public:

  Int_Display();
  virtual ~Int_Display() {}

  // Value.
  
  void set(int);

private:

  text::Display * _widget;
};

// Integer slider widget.

class Int_Slider : public widget::Widget, public Int_Base {
public:

  Int_Slider();
  virtual ~Int_Slider() {}
  
  // Value.
  
  virtual void default_valid(bool);
  bool default_valid() const { return Int_Base::default_valid(); }
  
  // Label.
  
  void label(widget::Widget *);
  widget::Widget * label() const { return _label; }
  virtual Item * label_align() const { return _label; }
  
  // Widget.

  virtual int handle(int);
  
  // Layout.
  
  virtual Item * group_align();
  
private:

  DJV_CALLBACK(Int_Slider, widget_callback, const Str &);
  DJV_CALLBACK(Int_Slider, inc_callback, bool);
  DJV_CALLBACK(Int_Slider, dec_callback, bool);
  DJV_CALLBACK(Int_Slider, slider_callback, int);
  DJV_CALLBACK(Int_Slider, default_callback, bool);
  
  void value_update();
  void range_update();
  
  text::Edit * _widget;
  slider::Int * _slider;
  widget::Widget * _label;
  button::Tool * _default_widget;
  layout::Horizontal * _layout;
};

}
}
}

#endif

