/*------------------------------------------------------------------------------
 lib/djv_ui/shuttle.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "shuttle.h"
#include <djv_ui/icon.h>
#include <FL/Fl.H>
#include <FL/Fl_Image.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace shuttle {

Widget::Widget(const Str & prefix) :
  _value(0.0),
  _pressed(false)
{
  _icon[0] = icon::get(prefix + "shuttle_0");
  _icon[1] = icon::get(prefix + "shuttle_1");
  _icon[2] = icon::get(prefix + "shuttle_2");
  _icon[3] = icon::get(prefix + "shuttle_3");
  _icon[4] = icon::get(prefix + "shuttle_4");
  _icon[5] = icon::get(prefix + "shuttle_5");
  _icon[6] = icon::get(prefix + "shuttle_6");
  _icon[7] = icon::get(prefix + "shuttle_7");

  size_policy(FIXED, FIXED);
}

int Widget::handle(int in)
{
  const Vec2i mouse(Fl::event_x(), Fl::event_y());
  switch (in) {

    case FL_PUSH:
      _value = 0.0;
      _pressed = true;
      _mouse_td = mouse;
      redraw();
      sig.emit(true);
      return 1;
      
    case FL_RELEASE:
      _value = 0;
      _pressed = false;
      redraw();
      sig.emit(false);
      return 1;
    
    case FL_DRAG: {
      const int tmp = mouse.x - _mouse_td.x;
      if (tmp != _value) {
        _value = double(tmp);
        redraw();
        value_sig.emit(int(_value / 5.0));
      }
    } return 1;
  }
  return widget::Widget::handle(in);
}

void Widget::draw()
{
  widget::Base::draw();

  const Box2i geom = this->geom();
  
  fl_color(FL_BACKGROUND_COLOR);
  fl_rectf(geom.x, geom.y, geom.w, geom.h);
  if (_pressed)
    fl_draw_box(FL_THIN_DOWN_FRAME, geom.x, geom.y, geom.w, geom.h,
      FL_BACKGROUND_COLOR);
  
  const int i = math::mod(int(_value / 5.0), 8);
  _icon[i]->draw(
    geom.x + (geom.w - _icon[0]->w()) / 2,
    geom.y + (geom.h - _icon[0]->h()) / 2
  );
}

Vec2i Widget::size_hint() const
  { return Vec2i(_icon[0]->w(), _icon[0]->h()) + 2 * 2; }

}
}
}
