/*------------------------------------------------------------------------------
 lib/djv_ui/widget_base.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG
//#undef DJV_DBG
#define _DBG "lib/djv_ui/widget_base.cxx"

#include "widget.h"
#include <djv_ui/style.h>
#include <djv_ui/app.h>
#include <FL/Fl.H>
#include <FL/fl_draw.H>

namespace djv {
namespace ui {
namespace widget {

Base::Base() :
  Item(WIDGET),
  _text_font(style::style()->font()), _text_color(FL_FOREGROUND_COLOR)
{
  //DJV_DBG_OBJ2(_DBG, "Base::Base");
  
  app::app()->widget_add(this);
}

Base::~Base()
{
  //DJV_DBG_OBJ2(_DBG, "Base::~Base");
}

void Base::text_font(const Fl_Font & in)
{
  _text_font = in;
}

void Base::text_color(const Fl_Color & in)
{
  _text_color = in;
}

void Base::tooltip(const Str & in)
{
  _tooltip = in;
}

int Base::handle(int in)
{
  switch (in) {
  
    case FL_KEYBOARD:
      switch (Fl::event_key()) {
      
        // XXX Disable wacky FLTK keyboard navigation.
        
        case FL_Left:
        case FL_Right:
        case FL_Up:
        case FL_Down: return -1;
      }
  }
  return 0;
}

namespace {

void draw_item(layout::Item * in)
{
  const Box2i & geom = in->geom();
  
  const int x = geom.x, y = geom.y;
  const int w = geom.w, h = geom.h;
  switch (in->type()) {
    case layout::Item::WIDGET:
      fl_color(FL_RED);
      fl_rect(x + 0, y + 0, w - 0, h - 0);
      break;
    case layout::Item::LAYOUT:
      fl_color(FL_BLUE);
      fl_rect(x + 0, y + 0, w - 0, h - 0);
      break;
    case layout::Item::SPACER:
      fl_color(FL_CYAN);
      fl_rectf(x + 0, y + 0, w - 0, h - 0);
      break;
  }
}

void draw_layout(layout::Layout * in)
{
  const List<layout::Item *> & list = in->list();
  for (uint i = 0; i < list.size(); ++i)
    if (layout::Item::LAYOUT == list[i]->type())
      draw_layout(static_cast<layout::Layout *>(list[i]));
    else
      draw_item(list[i]);
  draw_item(in);
}

}

void Base::draw()
{
  //if (_layout.get()) draw_layout(_layout.get());
  //draw_item(this);
}

void Base::widget_del()
{
  DJV_DBG_OBJ2(_DBG, str::Format("Base::widget_del(%1)").arg(dbg_name()));

  app::app()->widget_del(this);

  _layout.reset();

  Fl_Widget * widget = dynamic_cast<Fl_Widget *>(this);
  DJV_ASSERT(widget);
  if (widget->parent()) {
    DJV_DBG_PRN("fltk remove");
    widget->parent()->remove(widget);
    widget->parent(0);
  }
  Fl::delete_widget(widget);

  Callback_Base::del();
}

void Base::layout(layout::Layout * in)
{
  _layout = std::auto_ptr<layout::Layout>(in);
  _layout->parent(this);
}

void Base::dirty()
{
  Item::dirty();
  if (_layout.get()) {
    _layout->dirty();
    _size_hint = _layout->size_hint();
  } else
    _size_hint = Item::size_hint();
}

void Base::resize(int x, int y, int w, int h)
{
  //DJV_DBG_OBJ2(_DBG, str::Format("Base::geom(%1)").arg(dbg_name()));
  //DJV_DBG_PRN("in = " << Box2i(x, y, w, h));

  Item::resize(x, y, w, h);
  if (_layout.get()) _layout->resize(x, y, w, h);
}

}
}
}
