/*------------------------------------------------------------------------------
 plugin/djv_jpeg/ui.cxx

 Copyright (c) 2004-2007 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "ui.h"
#include <djv_ui/style.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/num_int.h>

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_ui_image_save()
  { return new Save_Widget; }
}

Save_Widget::Save_Widget() :
  _plugin(0)
{
  // Widgets.
  
  ui::group::Box * quality_group = new ui::group::Box("Quality");
  _quality_widget = new ui::num::Int_Slider;
  _quality_widget->range(0, 100);
  ui::label::Text * quality_text = new ui::label::Text(
    "File compression quality.");
  
  // Layout.
  
  ui::layout::Vertical * layout = new ui::layout::Vertical(this);
  layout->spacing(ui::style::style()->spacing_large());
  
  layout->add(quality_group);
  ui::layout::Vertical * group_layout = new ui::layout::Vertical(quality_group);
  group_layout->add(_quality_widget);
  group_layout->add_spacer();
  group_layout->add_spacer(); // XXX
  group_layout->add(quality_text);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _quality_widget->sig.set(this, quality_callback);
}

void Save_Widget::set(image::io::Plugin * in)
{
  _plugin = in;
  widget_update();
}

void Save_Widget::quality_callback(int in)
{
  if (! _plugin) return;
  _options.quality = in;
  plugin_update();
  sig.emit(_plugin);
}

void Save_Widget::plugin_update()
{
  callbacks(false);

  if (_plugin) {
    List<Str> tmp;
    tmp = str::serialize(_options.quality);
    _plugin->option(_plugin->option_list()[::Save::OPTION_QUALITY], &tmp);
  }

  callbacks(true);
}

void Save_Widget::widget_update()
{
  callbacks(false);
  
  if (_plugin) {
    List<Str> tmp;
    tmp = _plugin->option(_plugin->option_list()[::Save::OPTION_QUALITY]);
    str::serialize(&tmp, &_options.quality);
  }
  
  _quality_widget->set(_options.quality);
  
  callbacks(true);
}

