//#define DJV_DBG

#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/color.h>
#include <djv_ui/app.h>

using namespace djv;

class App : public ui::app::App {
public:

  App(int & argc, char ** argv);
  ~App();

private:

  DJV_CALLBACK(App, widget_callback, const image::Color &);
  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  ui::color::Swatch * _swatch;
  ui::window::Widget * _window;
};

App::App(int & argc, char ** argv) :
  ui::app::App("color_widget", argc, argv),
  _window(0)
{
  ui::color::Widget * widget = new ui::color::Widget;
  _swatch = new ui::color::Swatch;
  _window = new ui::window::Widget(name());
  
  ui::layout::Horizontal * layout = new ui::layout::Horizontal(_window);
  layout->add(_swatch);
  ui::layout::Vertical * v_layout = new ui::layout::Vertical(layout);
  v_layout->margin(0);
  layout->stretch(v_layout);
  v_layout->add(widget);
  
  _window->size(_window->size_hint());
  _window->show();
  
  widget->sig.set(this, widget_callback);
  _window->close_sig.set(this, exit_callback);

  valid(true);
}

App::~App()
{
  if (_window) _window->widget_del();
}

void App::widget_callback(const image::Color & in)
{
  _swatch->set(in);
  
  print(str::Format("value = %1").
    arg(str::join(image::color::serialize(in), " ")));
}

int main(int argc, char ** argv)
{
  return App(argc, argv).exec();
}
